#!/usr/bin/env python
"""
This HTTP server can be run on a server, and redirects the SAMLResponse to 127.0.0.1 so the command can capture it
"""

from http.server import BaseHTTPRequestHandler, HTTPServer
import logging


class RedirectServerHandler(BaseHTTPRequestHandler):
    def do_POST(self):
        self.send_response(307)
        self.send_header("location", "http://127.0.0.1:4589/")
        self.end_headers()


def start_redirect_server(port):
    logging.basicConfig(level=logging.INFO)
    server_address = ("", port)
    httpd = HTTPServer(server_address, RedirectServerHandler)
    logging.info("Starting http redirect server on: %s", port)
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        pass
    httpd.server_close()
    logging.info("Stopping http redirect server")



import os
import unittest

import mock
from nose.tools import nottest

from aws_saml_auth import resolve_config, parse_args


class TestProfileProcessing(unittest.TestCase):
    def test_default(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual("sts", config.profile)

    def test_cli_param_supplied(self):
        args = parse_args(["-p", "profile"])
        config = resolve_config(args)
        self.assertEqual("profile", config.profile)

    @mock.patch.dict(os.environ, {"AWS_PROFILE": "mytemp"})
    def test_with_environment(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual("mytemp", config.profile)

        args = parse_args(["-p", "profile"])
        config = resolve_config(args)
        self.assertEqual("profile", config.profile)


class TestDurationProcessing(unittest.TestCase):
    def test_default(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual(43200, config.duration)

    def test_cli_param_supplied(self):
        args = parse_args(["-d", "500"])
        config = resolve_config(args)
        self.assertEqual(500, config.duration)

    def test_invalid_cli_param_supplied(self):

        with self.assertRaises(SystemExit):
            args = parse_args(["-d", "blart"])
            resolve_config(args)

    @mock.patch.dict(os.environ, {"ASA_DURATION": "3000"})
    def test_with_environment(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual(3000, config.duration)

        args = parse_args(["-d", "500"])
        config = resolve_config(args)
        self.assertEqual(500, config.duration)


class TestLoginUrlProcessing(unittest.TestCase):
    def test_default(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual(None, config.login_url)

    def test_cli_param_supplied(self):
        args = parse_args(["-L", "kjl2342"])
        config = resolve_config(args)
        self.assertEqual("kjl2342", config.login_url)

    @mock.patch.dict(os.environ, {"ASA_LOGIN_URL": "adsfasf233423"})
    def test_with_environment(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual("adsfasf233423", config.login_url)

        args = parse_args(["-L", "kjl2342"])
        config = resolve_config(args)
        self.assertEqual("kjl2342", config.login_url)


class TestRegionProcessing(unittest.TestCase):
    @nottest
    def test_default(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual(None, config.region)

    def test_cli_param_supplied(self):
        args = parse_args(["--region", "ap-southeast-4"])
        config = resolve_config(args)
        self.assertEqual("ap-southeast-4", config.region)

    @mock.patch.dict(os.environ, {"AWS_DEFAULT_REGION": "ap-southeast-9"})
    def test_with_environment(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual("ap-southeast-9", config.region)

        args = parse_args(["--region", "ap-southeast-4"])
        config = resolve_config(args)
        self.assertEqual("ap-southeast-4", config.region)


class TestRoleProcessing(unittest.TestCase):
    def test_default(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual(None, config.role_arn)

    def test_cli_param_supplied(self):
        args = parse_args(["-r", "role1234"])
        config = resolve_config(args)
        self.assertEqual("role1234", config.role_arn)

    @mock.patch.dict(os.environ, {"ASA_ROLE_ARN": "4567-role"})
    def test_with_environment(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual("4567-role", config.role_arn)


class TestAskRoleProcessing(unittest.TestCase):
    def test_default(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertTrue(config.ask_role)

    def test_cli_param_supplied(self):
        args = parse_args(["-a"])
        config = resolve_config(args)
        self.assertTrue(config.ask_role)

    @nottest
    @mock.patch.dict(os.environ, {"ASA_ASK_ROLE": "true"})
    def test_with_environment(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertTrue(config.ask_role)


class TestResolveAliasesProcessing(unittest.TestCase):
    def test_default(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertTrue(config.resolve_aliases)

    def test_cli_param_supplied(self):
        args = parse_args(["--no-resolve-aliases"])
        config = resolve_config(args)
        self.assertFalse(config.resolve_aliases)

    @nottest
    @mock.patch.dict(os.environ, {"ASA_NO_RESOLVE_AWS_ALIASES": "true"})
    def test_with_environment(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertFalse(config.resolve_aliases)


class TestAccountProcessing(unittest.TestCase):
    @nottest
    def test_default(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual(None, config.account)

    def test_cli_param_supplied(self):
        args = parse_args(["--account", "123456789012"])
        config = resolve_config(args)
        self.assertEqual("123456789012", config.account)

    @mock.patch.dict(os.environ, {"ASA_AWS_ACCOUNT": "123456789012"})
    def test_with_environment(self):
        args = parse_args([])
        config = resolve_config(args)
        self.assertEqual("123456789012", config.account)

        args = parse_args(["--region", "123456789012"])
        config = resolve_config(args)
        self.assertEqual("123456789012", config.account)



#!/usr/bin/env python

import unittest
from random import randint

import configparser

from aws_saml_auth import configuration


class TestConfigurationPersistence(unittest.TestCase):
    def setUp(self):
        self.c = configuration.Configuration()

        # Pick a profile name that is clear it's for testing. We'll delete it
        # after, but in case something goes wrong we don't want to use
        # something that could clobber user input.
        self.c.profile = "aws_saml_auth_test_{}".format(randint(100, 999))

        self.c.region = "us-east-1"
        self.c.ask_role = False
        self.c.duration = 1234
        self.c.role_arn = "arn:aws:iam::sample_arn"
        self.c.login_url = "sample_login_url"
        self.c.raise_if_invalid()
        self.c.write(None)
        self.c.account = "123456789012"

        self.config_parser = configparser.RawConfigParser()
        self.config_parser.read(self.c.config_file)

    def tearDown(self):
        section_name = configuration.Configuration.config_profile(self.c.profile)
        self.config_parser.remove_section(section_name)
        with open(self.c.config_file, "w") as config_file:
            self.config_parser.write(config_file)

    def test_creating_new_profile(self):
        profile_string = configuration.Configuration.config_profile(self.c.profile)
        self.assertTrue(self.config_parser.has_section(profile_string))
        self.assertEqual(
            self.config_parser[profile_string].get("asa.role_arn"), self.c.role_arn
        )
        self.assertEqual(
            self.config_parser[profile_string].get("asa.login_url"), self.c.login_url
        )
        self.assertEqual(
            self.config_parser[profile_string].get("region"), self.c.region
        )
        self.assertEqual(
            self.config_parser[profile_string].getboolean("asa.ask_role"),
            self.c.ask_role,
        )
        self.assertEqual(
            self.config_parser[profile_string].getint("asa.duration"), self.c.duration
        )

    def test_can_read_all_values(self):
        test_configuration = configuration.Configuration()
        test_configuration.read(self.c.profile)

        test_configuration.raise_if_invalid()

        self.assertEqual(test_configuration.profile, self.c.profile)
        self.assertEqual(test_configuration.role_arn, self.c.role_arn)
        self.assertEqual(test_configuration.login_url, self.c.login_url)
        self.assertEqual(test_configuration.region, self.c.region)
        self.assertEqual(test_configuration.ask_role, self.c.ask_role)
        self.assertEqual(test_configuration.duration, self.c.duration)



#!/usr/bin/env python

import base64
import boto3
import os
import re
import json

from datetime import datetime
from threading import Thread

from botocore.exceptions import ClientError, ProfileNotFound
from lxml import etree


class ExpectedAmazonException(Exception):
    def __init__(self, *args):
        super(ExpectedAmazonException, self).__init__(*args)


class Amazon:
    def __init__(self, config, saml_xml):
        self.config = config
        self.saml_xml = saml_xml
        if config.token_cache:
            self.__token = {"Credentials": config.token_cache}
            self.__roles = [config.role_arn]
        else:
            self.__token = None

    @property
    def sts_client(self):
        try:
            profile = os.environ.get("AWS_PROFILE")
            if profile is not None:
                del os.environ["AWS_PROFILE"]
            client = boto3.client("sts", region_name=self.config.region)
            if profile is not None:
                os.environ["AWS_PROFILE"] = profile
            return client
        except ProfileNotFound as ex:
            raise ExpectedAmazonException("Error : {}.".format(ex))

    @property
    def base64_encoded_saml(self):
        return base64.b64encode(self.saml_xml).decode("utf-8")

    @property
    def token(self):
        if self.__token is None:
            self.__token = self.assume_role(
                self.config.role_arn,
                self.config.provider,
                self.base64_encoded_saml,
                self.config.duration,
            )
        return self.__token

    @property
    def access_key_id(self):
        return self.token["Credentials"]["AccessKeyId"]

    @property
    def secret_access_key(self):
        return self.token["Credentials"]["SecretAccessKey"]

    @property
    def session_token(self):
        return self.token["Credentials"]["SessionToken"]

    @property
    def expiration(self):
        return self.token["Credentials"]["Expiration"]

    def print_export_line(self):
        export_template = "export AWS_ACCESS_KEY_ID='{}' AWS_SECRET_ACCESS_KEY='{}' AWS_SESSION_TOKEN='{}' AWS_SESSION_EXPIRATION='{}'"

        formatted = export_template.format(
            self.access_key_id,
            self.secret_access_key,
            self.session_token,
            self.expiration.strftime("%Y-%m-%dT%H:%M:%S%z"),
        )

        print(formatted)

    def print_credential_process(self):
        print(
            json.dumps(
                {
                    "Version": 1,
                    "AccessKeyId": self.access_key_id,
                    "SecretAccessKey": self.secret_access_key,
                    "SessionToken": self.session_token,
                    "Expiration": self.expiration.strftime("%Y-%m-%dT%H:%M:%S%z"),
                }
            )
        )

    @property
    def roles(self):
        assert self.saml_xml is not None, "Cannot load roles without saml."
        doc = etree.fromstring(self.saml_xml)
        roles = {}
        for x in doc.xpath(
            '//*[@Name = "https://aws.amazon.com/SAML/Attributes/Role"]//text()'
        ):
            if "arn:aws:iam:" in x or "arn:aws-us-gov:iam:" in x:
                res = x.split(",")
                roles[res[0]] = res[1]
        return roles

    def assume_role(
        self, role, principal, saml_assertion, duration=None, auto_duration=True
    ):
        sts_call_vars = {
            "RoleArn": role,
            "PrincipalArn": principal,
            "SAMLAssertion": saml_assertion,
        }

        # Try the maximum duration of 12 hours, if it fails try to use the
        # maximum duration indicated by the error
        if self.config.auto_duration and auto_duration:
            sts_call_vars["DurationSeconds"] = self.config.max_duration
            try:
                res = self.sts_client.assume_role_with_saml(**sts_call_vars)
            except ClientError as err:
                if err.response.get("Error", []).get(
                    "Code"
                ) == "ValidationError" and err.response.get("Error", []).get("Message"):
                    m = re.search(
                        "Member must have value less than or equal to ([0-9]{3,5})",
                        err.response["Error"]["Message"],
                    )
                    if m is not None and m.group(1):
                        new_duration = int(m.group(1))
                        return self.assume_role(
                            role,
                            principal,
                            saml_assertion,
                            duration=new_duration,
                            auto_duration=False,
                        )
                # Unknown error or no max time returned in error message
                raise
        elif duration:
            sts_call_vars["DurationSeconds"] = duration

        res = self.sts_client.assume_role_with_saml(**sts_call_vars)

        return res

    def resolve_aws_aliases(self, roles):
        def resolve_aws_alias(role, principal, aws_dict):
            try:
                session = boto3.session.Session(region_name=self.config.region)

                sts = session.client("sts")
                saml = sts.assume_role_with_saml(
                    RoleArn=role,
                    PrincipalArn=principal,
                    SAMLAssertion=self.base64_encoded_saml,
                )

                iam = session.client(
                    "iam",
                    aws_access_key_id=saml["Credentials"]["AccessKeyId"],
                    aws_secret_access_key=saml["Credentials"]["SecretAccessKey"],
                    aws_session_token=saml["Credentials"]["SessionToken"],
                )

                response = iam.list_account_aliases()
                account_alias = response["AccountAliases"][0]
                aws_dict[role.split(":")[4]] = account_alias
            except:
                aws_dict[role.split(":")[4]] = role.split(":")[4]

        threads = []
        aws_id_alias = {}
        for number, (role, principal) in enumerate(roles.items()):
            t = Thread(target=resolve_aws_alias, args=(role, principal, aws_id_alias))
            t.start()
            threads.append(t)

        for t in threads:
            t.join()

        return aws_id_alias

    @staticmethod
    def is_valid_saml_assertion(saml_xml):
        if saml_xml is None:
            return False

        try:
            doc = etree.fromstring(saml_xml)
            conditions = list(
                doc.iter(tag="{urn:oasis:names:tc:SAML:2.0:assertion}Conditions")
            )
            not_before_str = conditions[0].get("NotBefore")
            not_on_or_after_str = conditions[0].get("NotOnOrAfter")

            now = datetime.utcnow()
            not_before = datetime.strptime(not_before_str, "%Y-%m-%dT%H:%M:%S.%fZ")
            not_on_or_after = datetime.strptime(
                not_on_or_after_str, "%Y-%m-%dT%H:%M:%S.%fZ"
            )

            if not_before <= now < not_on_or_after:
                return True
            else:
                return False
        except Exception:
            return False



#!/usr/bin/env python

import os

import botocore.session
import filelock
import logging
from datetime import datetime
from dateutil import tz
import hashlib

try:
    from backports import configparser
except ImportError:
    import configparser

from aws_saml_auth import util
from aws_saml_auth import amazon


class Configuration(object):
    def __init__(self, **kwargs):
        self.options = {}
        self.__boto_session = botocore.session.Session()

        # Set up some defaults. These can be overridden as fit.
        self.ask_role = False
        self.duration = self.max_duration
        self.auto_duration = False
        self.login_url = None
        self.profile = "sts"
        self.region = None
        self.role_arn = None
        self.__saml_cache = None
        self.__token_cache = None
        self.resolve_aliases = True
        self.print_creds = False
        self.credential_process = False
        self.quiet = False
        self.account = ""
        self.port = 8000

    # For the "~/.aws/config" file, we use the format "[profile testing]"
    # for the 'testing' profile. The credential file will just be "[testing]"
    # in that case. See https://docs.aws.amazon.com/cli/latest/userguide/cli-multiple-profiles.html
    # for more information.
    @staticmethod
    def config_profile(profile):
        if str(profile).lower() == "default":
            return profile
        else:
            return "profile {}".format(str(profile))

    @property
    def max_duration(self):
        return 43200

    @property
    def credentials_file(self):
        return os.path.expanduser(
            self.__boto_session.get_config_variable("credentials_file")
        )

    @property
    def config_file(self):
        return os.path.expanduser(
            self.__boto_session.get_config_variable("config_file")
        )

    @property
    def saml_cache_file(self):
        assert (
            self.login_url is not None
        ), "Cannot look for smal cache file if no login url"

        return self.credentials_file.replace(
            "credentials",
            "saml_cache_%s.xml"
            % hashlib.sha1(self.login_url.encode("utf-8")).hexdigest(),
        )

    def ensure_config_files_exist(self):
        for file in [self.config_file, self.credentials_file]:
            directory = os.path.dirname(file)
            if not os.path.exists(directory):
                os.mkdir(directory, 0o700)
            if not os.path.exists(file):
                util.Util.touch(file)

    # Will return a SAML cache, ONLY if it's valid. If invalid or not set, will
    # return None. If the SAML cache isn't valid, we'll remove it from the
    # in-memory object. On the next write(), it will be purged from disk.
    @property
    def saml_cache(self):
        if not amazon.Amazon.is_valid_saml_assertion(self.__saml_cache):
            logging.debug("%s: Invalid saml cache", __name__)
            self.__saml_cache = None

        return self.__saml_cache

    @saml_cache.setter
    def saml_cache(self, value):
        self.__saml_cache = value

    # Will return a credential cache, ONLY if it's valid.
    @property
    def token_cache(self):
        if self.__token_cache is not None:
            if (
                self.__token_cache["Expiration"] is None
                or self.__token_cache["Expiration"] <= datetime.now(tz.UTC)
                or self.__token_cache["AccessKeyId"] is None
                or self.__token_cache["SecretAccessKey"] is None
            ):
                logging.debug("%s: Invalid token cache", __name__)
                self.__token_cache = None

        return self.__token_cache

    # Will raise exceptions if the configuration is invalid, otherwise returns
    # None. Use this at any point to validate the configuration is in a good
    # state. There are no checks here regarding SAML caching, as that's just a
    # user-performance improvement, and an invalid cache isn't an invalid
    # configuration.
    def raise_if_invalid(self):
        # ask_role
        assert (
            self.ask_role.__class__ is bool
        ), "Expected ask_role to be a boolean. Got {}.".format(self.ask_role.__class__)

        # duration
        assert (
            self.duration.__class__ is int
        ), "Expected duration to be an integer. Got {}.".format(self.duration.__class__)
        assert (
            self.duration >= 900
        ), "Expected duration to be greater than or equal to 900. Got {}.".format(
            self.duration
        )
        assert (
            self.duration <= self.max_duration
        ), "Expected duration to be less than or equal to max_duration ({}). Got {}.".format(
            self.max_duration, self.duration
        )

        # auto_duration
        assert (
            self.auto_duration.__class__ is bool
        ), "Expected auto_duration to be a boolean. Got {}.".format(
            self.auto_duration.__class__
        )

        # profile
        assert (
            self.profile.__class__ is str
        ), "Expected profile to be a string. Got {}.".format(self.profile.__class__)

        # region
        assert (
            self.region.__class__ is str
        ), "Expected region to be a string. Got {}.".format(self.region.__class__)

        # login_url
        assert (
            self.login_url is not None
        ), "Expected login_url to be set to non-None value."

        # role_arn (Can be blank, we'll just prompt)
        if self.role_arn is not None:
            assert (
                self.role_arn.__class__ is str
            ), "Expected role_arn to be None or a string. Got {}.".format(
                self.role_arn.__class__
            )
            assert (
                "arn:aws:iam::" in self.role_arn
                or "arn:aws-us-gov:iam::" in self.role_arn
            ), "Expected role_arn to contain 'arn:aws:iam::'. Got '{}'.".format(
                self.role_arn
            )

        # quiet
        assert (
            self.quiet.__class__ is bool
        ), "Expected quiet to be a boolean. Got {}.".format(self.quiet.__class__)

        # account
        assert (
            self.account.__class__ is str
        ), "Expected account to be string. Got {}".format(self.account.__class__)

        # port
        assert (
            self.port.__class__ is int
        ), "Expected port to be an integer. Got {}.".format(self.port.__class__)

    # Write the configuration (and credentials) out to disk. This allows for
    # regular AWS tooling (aws cli and boto) to use the credentials in the
    # profile the user specified.
    def write(self, amazon_object):
        self.ensure_config_files_exist()

        assert (
            self.profile is not None
        ), "Can not store config/credentials if the AWS_PROFILE is None."

        config_file_lock = filelock.FileLock(self.config_file + ".lock")
        config_file_lock.acquire()
        try:
            # Write to the configuration file
            profile = Configuration.config_profile(self.profile)
            config_parser = configparser.RawConfigParser()
            config_parser.read(self.config_file)
            if not config_parser.has_section(profile):
                config_parser.add_section(profile)
            config_parser.set(profile, "region", self.region)
            config_parser.set(profile, "asa.ask_role", self.ask_role)
            config_parser.set(profile, "asa.duration", self.duration)
            config_parser.set(profile, "asa.login_url", self.login_url)
            config_parser.set(profile, "asa.role_arn", self.role_arn)

            with open(self.config_file, "w+") as f:
                config_parser.write(f)
        finally:
            config_file_lock.release()

            # Write to the credentials file (only if we have credentials)
            if amazon_object is not None:
                credentials_file_lock = filelock.FileLock(
                    self.credentials_file + ".lock"
                )
                credentials_file_lock.acquire()
                try:
                    credentials_parser = configparser.RawConfigParser()
                    credentials_parser.read(self.credentials_file)
                    if not credentials_parser.has_section(self.profile):
                        credentials_parser.add_section(self.profile)
                    credentials_parser.set(
                        self.profile, "aws_access_key_id", amazon_object.access_key_id
                    )
                    credentials_parser.set(
                        self.profile,
                        "aws_secret_access_key",
                        amazon_object.secret_access_key,
                    )
                    credentials_parser.set(
                        self.profile, "aws_security_token", amazon_object.session_token
                    )
                    credentials_parser.set(
                        self.profile,
                        "aws_session_expiration",
                        amazon_object.expiration.strftime("%Y-%m-%dT%H:%M:%S%z"),
                    )
                    credentials_parser.set(
                        self.profile, "aws_session_token", amazon_object.session_token
                    )

                    with open(self.credentials_file, "w+") as f:
                        credentials_parser.write(f)
                finally:
                    credentials_file_lock.release()

    def write_saml_cache(self):
        self.ensure_config_files_exist()

        if self.__saml_cache is not None:
            saml_cache_file_lock = filelock.FileLock(self.saml_cache_file + ".lock")
            saml_cache_file_lock.acquire()
            try:
                with open(self.saml_cache_file, "w") as f:
                    f.write(self.__saml_cache.decode("utf-8"))
            finally:
                saml_cache_file_lock.release()

    def write_token_cache(self, amazon_object):
        assert (
            self.profile is not None
        ), "Can not store config/credentials if the AWS_PROFILE is None."
        assert (
            amazon_object is not None
        ), "Can not store config/credentials if the amazon_object is None."

        credentials_file_lock = filelock.FileLock(self.credentials_file + ".lock")
        credentials_file_lock.acquire()
        try:
            credentials_parser = configparser.RawConfigParser()
            credentials_parser.read(self.credentials_file)
            if not credentials_parser.has_section(self.profile):
                credentials_parser.add_section(self.profile)
            credentials_parser.set(
                self.profile, "asa.aws_access_key_id", amazon_object.access_key_id
            )
            credentials_parser.set(
                self.profile,
                "asa.aws_secret_access_key",
                amazon_object.secret_access_key,
            )
            credentials_parser.set(
                self.profile,
                "asa.aws_session_expiration",
                amazon_object.expiration.strftime("%Y-%m-%dT%H:%M:%S%z"),
            )
            credentials_parser.set(
                self.profile, "asa.aws_session_token", amazon_object.session_token
            )

            with open(self.credentials_file, "w+") as f:
                credentials_parser.write(f)
        finally:
            credentials_file_lock.release()

    # Read from the configuration file and override ALL values currently stored
    # in the configuration object. As this is potentially destructive, it's
    # important to only run this in the beginning of the object initialization.
    # We do not read AWS credentials, as this tool's use case is to obtain
    # them.
    def read(self, profile):
        self.ensure_config_files_exist()

        # Shortening Convenience functions
        coalesce = util.Util.coalesce
        unicode_to_string = util.Util.unicode_to_string_if_needed

        profile_string = Configuration.config_profile(profile)
        config_parser = configparser.RawConfigParser()
        config_parser.read(self.config_file)

        if config_parser.has_section(profile_string):
            self.profile = profile

            # Ask Role
            read_ask_role = config_parser[profile_string].getboolean(
                "asa.ask_role", None
            )
            self.ask_role = coalesce(read_ask_role, self.ask_role)

            # Duration
            read_duration = config_parser[profile_string].getint("asa.duration", None)
            self.duration = coalesce(read_duration, self.duration)

            # Login URL
            read_login_url = unicode_to_string(
                config_parser[profile_string].get("asa.login_url", None)
            )
            self.login_url = coalesce(read_login_url, self.login_url)

            # Region
            read_region = unicode_to_string(
                config_parser[profile_string].get("region", None)
            )
            self.region = coalesce(read_region, self.region)

            # Role ARN
            read_role_arn = unicode_to_string(
                config_parser[profile_string].get("asa.role_arn", None)
            )
            self.role_arn = coalesce(read_role_arn, self.role_arn)

            # Account
            read_account = unicode_to_string(
                config_parser[profile_string].get("account", None)
            )
            self.account = coalesce(read_account, self.account)

    def read_saml_cache(self):
        if self.login_url is None:
            return
        try:
            with open(self.saml_cache_file, "r") as f:
                self.__saml_cache = f.read().encode("utf-8")
        except IOError as ex:
            logging.info("%s: SAML cache failed to read: %s", __name__, ex)
            pass

    def read_token_cache(self):
        assert (
            self.profile is not None
        ), "Can not store config/credentials if the AWS_PROFILE is None."

        # Shortening Convenience functions
        unicode_to_string = util.Util.unicode_to_string_if_needed

        credentials_parser = configparser.RawConfigParser()
        credentials_parser.read(self.credentials_file)

        if credentials_parser.has_section(self.profile):
            token = {}
            token["AccessKeyId"] = unicode_to_string(
                credentials_parser[self.profile].get("asa.aws_access_key_id", None)
            )
            token["SecretAccessKey"] = unicode_to_string(
                credentials_parser[self.profile].get("asa.aws_secret_access_key", None)
            )
            token["SessionToken"] = unicode_to_string(
                credentials_parser[self.profile].get("asa.aws_session_token", None)
            )
            read_expiration = unicode_to_string(
                credentials_parser[self.profile].get("asa.aws_session_expiration", None)
            )
            if read_expiration is not None:
                token["Expiration"] = datetime.strptime(
                    read_expiration, "%Y-%m-%dT%H:%M:%S%z"
                ).replace(tzinfo=tz.UTC)
            else:
                token["Expiration"] = None
            self.__token_cache = token



#!/usr/bin/env python
"""
This HTTP server for capturing the SAMLResponse that is redirected to 127.0.0.1
"""

from http.server import BaseHTTPRequestHandler, HTTPServer
import logging

from aws_saml_auth import util


class LoginServer(HTTPServer):
    post_data = {}


class LoginServerHandler(BaseHTTPRequestHandler):
    def _set_response(self):
        self.send_response(200)
        self.send_header("content-type", "text/html")
        self.end_headers()
        self.wfile.write(
            """
           <html>
           <head><title>Success</title></head>
           <body>
           Check your console
           <script>window.close()</script>
           </body>
           </html>
        """.encode(
                "utf-8"
            )
        )

    def do_POST(self):
        self.server.post_data = util.Util.parse_post(self)
        logging.debug(
            "POST request,\nPath: %s\nHeaders:\n%s\n\nBody:\n%s\n",
            str(self.path),
            str(self.headers),
            self.server.post_data,
        )

        self._set_response()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function

import base64
import logging
import webbrowser

from aws_saml_auth.login_server import LoginServerHandler, LoginServer


class ExpectedSamlException(Exception):
    def __init__(self, *args):
        super(ExpectedSamlException, self).__init__(*args)


class Saml:
    def __init__(self, config):
        """The Saml object opens the browser and catches the redirected response

        login_url: Another providers login url

        These are required to generated the correct login url
        """

        self.config = config

    def do_browser_saml(self):
        logging.warning("Opening url %s", self.login_url)
        webbrowser.open(self.login_url)
        saml_text = self._catch_saml()

        return base64.b64decode(saml_text)

    @staticmethod
    def _catch_saml(port=4589):
        server_address = ("", port)
        httpd = LoginServer(server_address, LoginServerHandler)
        logging.info("Starting http handler...\n")
        httpd.handle_request()

        assert (
            "SAMLResponse" in httpd.post_data
        ), "Expected post data to contain SAMLResponse."
        return httpd.post_data["SAMLResponse"][0]

    @property
    def login_url(self):
        if self.config.login_url is not None:
            return self.config.login_url

        raise ExpectedSamlException("No saml login url provided")



#!/usr/bin/env python
from __future__ import print_function

import argparse
import base64
import os
import sys
import logging

from six import print_ as print
from tzlocal import get_localzone

from aws_saml_auth import amazon
from aws_saml_auth import configuration
from aws_saml_auth import saml
from aws_saml_auth import util


def parse_args(args):
    parser = argparse.ArgumentParser(
        prog="aws-saml-auth",
        description="Acquire temporary AWS credentials via SAML",
    )

    main_group = parser.add_mutually_exclusive_group()
    main_group.add_argument(
        "--redirect-server",
        action="store_true",
        help="Run the redirect server on port ($PORT)",
    )
    main_group.add_argument(
        "-L", "--login-url", help="SAML Provider login url ($ASA_LOGIN_URL)"
    )
    parser.add_argument(
        "-R", "--region", help="AWS region endpoint ($AWS_DEFAULT_REGION)"
    )
    duration_group = parser.add_mutually_exclusive_group()
    duration_group.add_argument(
        "-d",
        "--duration",
        type=int,
        help="Credential duration in seconds (defaults to value of $ASA_DURATION, then falls back to 43200)",
    )
    duration_group.add_argument(
        "--auto-duration",
        action="store_true",
        help="Tries to use the longest allowed duration ($ASA_AUTO_DURATION=1)",
    )
    parser.add_argument(
        "-p",
        "--profile",
        help="AWS profile (defaults to value of $AWS_PROFILE, then falls back to 'sts')",
    )
    parser.add_argument(
        "-A", "--account", help="Filter for specific AWS account ($ASA_AWS_ACCOUNT)"
    )
    parser.add_argument("-q", "--quiet", action="store_true", help="Quiet output")
    parser.add_argument(
        "--saml-assertion",
        dest="saml_assertion",
        help="Base64 encoded SAML assertion to use",
    )
    parser.add_argument(
        "--no-saml-cache",
        dest="saml_cache",
        action="store_false",
        help="Do not cache the SAML Assertion",
    )
    print_group = parser.add_mutually_exclusive_group()
    print_group.add_argument(
        "--print-creds", action="store_true", help="Print Credentials"
    )
    print_group.add_argument(
        "--credential-process",
        action="store_true",
        help="Output suitable for aws cli credential_process ($ASA_CREDENTIAL_PROCESS=1)",
    )
    parser.add_argument(
        "--no-resolve-aliases",
        dest="resolve_aliases",
        action="store_false",
        help="Do not resolve AWS account aliases. ($ASA_NO_RESOLVE_ALIASES=1)",
    )
    parser.add_argument("--port", type=int, help="Port for the redirect server ($PORT)")

    role_group = parser.add_mutually_exclusive_group()
    role_group.add_argument(
        "-a",
        "--ask-role",
        action="store_true",
        help="Set true to always pick the role ($ASA_ASK_ROLE=1)",
    )
    role_group.add_argument(
        "-r", "--role-arn", help="The ARN of the role to assume ($ASA_ROLE_ARN)"
    )
    parser.add_argument(
        "-l",
        "--log",
        dest="log_level",
        choices=["debug", "info", "warn"],
        default="warn",
        help="Select log level (default: %(default)s)",
    )
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version="%(prog)s {version}".format(version="0.1.0"),
    )

    return parser.parse_args(args)


def exit_if_unsupported_python():
    if sys.version_info.major == 2 and sys.version_info.minor < 7:
        logging.critical(
            "%s requires Python 2.7 or higher. Please consider "
            "upgrading. Support for Python 2.6 and lower was "
            "dropped because this tool's dependencies dropped "
            "support.",
            __name__,
        )
        logging.critical(
            "For debugging, it appears you're running: %s", sys.version_info
        )
        logging.critical(
            "For more information, see: "
            "https://github.com/cevoaustralia/aws-google-auth/"
            "issues/41"
        )
        sys.exit(1)


def cli(cli_args):
    try:
        exit_if_unsupported_python()

        args = parse_args(args=cli_args)

        # Set up logging
        logging.getLogger().setLevel(getattr(logging, args.log_level.upper(), None))

        config = resolve_config(args)
        if args.redirect_server:
            from aws_saml_auth.redirect_server import start_redirect_server

            start_redirect_server(config.port)
            return

        process_auth(args, config)
    except amazon.ExpectedAmazonException as ex:
        print(ex)
        sys.exit(1)
    except saml.ExpectedSamlException as ex:
        print(ex)
        sys.exit(1)
    except KeyboardInterrupt:
        pass
    except Exception as ex:
        logging.exception(ex)


def resolve_config(args):

    # Shortening Convenience functions
    coalesce = util.Util.coalesce

    # Create a blank configuration object (has the defaults pre-filled)
    config = configuration.Configuration()

    # Have the configuration update itself via the ~/.aws/config on disk.
    # Profile (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.profile = coalesce(args.profile, os.getenv("AWS_PROFILE"), config.profile)

    # Now that we've established the profile, we can read the configuration and
    # fill in all the other variables.
    config.read(config.profile)

    # Ask Role (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.ask_role = (
        args.ask_role or os.getenv("ASA_ASK_ROLE") != None or config.ask_role != None
    )

    # Duration (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.duration = int(
        coalesce(args.duration, os.getenv("ASA_DURATION"), config.duration)
    )

    # Automatic duration (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.auto_duration = args.auto_duration or os.getenv("ASA_AUTO_DURATION") != None

    # Login URL (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.login_url = coalesce(
        args.login_url, os.getenv("ASA_LOGIN_URL"), config.login_url
    )

    # Region (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.region = coalesce(
        args.region, os.getenv("AWS_DEFAULT_REGION"), config.region
    )

    # ROLE ARN (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.role_arn = coalesce(
        args.role_arn, os.getenv("ASA_ROLE_ARN"), config.role_arn
    )

    # Resolve AWS aliases enabled (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.resolve_aliases = (
        coalesce(args.resolve_aliases, config.resolve_aliases)
        or os.getenv("ASA_NO_RESOLVE_ALIASES") != None
    )

    # Account (Option priority = ARGS, ENV_VAR, DEFAULT)
    config.account = coalesce(
        args.account, os.getenv("ASA_AWS_ACCOUNT"), config.account
    )

    config.print_creds = coalesce(args.print_creds, config.print_creds)

    # Quiet
    config.quiet = coalesce(args.quiet, config.quiet)

    config.port = int(coalesce(args.port, os.getenv("PORT"), config.port))

    config.credential_process = (
        args.credential_process or os.getenv("ASA_CREDENTIAL_PROCESS") != None
    )
    if config.credential_process:
        config.quiet = True
        config.ask_role = False
        config.read_token_cache()

    config.read_saml_cache()

    return config


def process_auth(args, config):
    if config.region is None:
        config.region = util.Util.get_input("AWS Region: ")
        logging.debug("%s: region is: %s", __name__, config.region)

    if config.login_url is None:
        config.login_url = util.Util.get_input("Login URL: ")
        logging.debug("%s: login url is: %s", __name__, config.login_url)

    # If there is a valid cache and the user opted to use it, use that instead
    # of prompting the user for input (it will also ignroe any set variables
    # such as username or sp_id and idp_id, as those are built into the SAML
    # response). The user does not need to be prompted for a password if the
    # SAML cache is used.
    if args.saml_assertion:
        saml_xml = base64.b64decode(args.saml_assertion)
    elif config.token_cache:
        saml_xml = None
    elif args.saml_cache and config.saml_cache:
        saml_xml = config.saml_cache
        logging.info("%s: SAML cache found", __name__)
    else:
        saml_client = saml.Saml(config)
        saml_xml = saml_client.do_browser_saml()

    # We now have a new SAML value that can get cached (If the user asked
    # for it to be)
    if args.saml_cache:
        config.saml_cache = saml_xml

    # The amazon_client now has the SAML assertion it needed (Either via the
    # cache or freshly generated). From here, we can get the roles and continue
    # the rest of the workflow regardless of cache.
    amazon_client = amazon.Amazon(config, saml_xml)
    if saml_xml is not None:
        roles = amazon_client.roles

        # Determine the provider and the role arn (if the the user provided isn't an option)
        if config.role_arn in roles and not config.ask_role:
            config.provider = roles[config.role_arn]
        else:
            if config.account and config.resolve_aliases:
                aliases = amazon_client.resolve_aws_aliases(roles)
                config.role_arn, config.provider = util.Util.pick_a_role(
                    roles, aliases, config.account
                )
            elif config.account:
                config.role_arn, config.provider = util.Util.pick_a_role(
                    roles, account=config.account
                )
            elif config.resolve_aliases:
                aliases = amazon_client.resolve_aws_aliases(roles)
                config.role_arn, config.provider = util.Util.pick_a_role(roles, aliases)
            else:
                config.role_arn, config.provider = util.Util.pick_a_role(roles)
        if not config.quiet:
            print("Assuming " + config.role_arn)
            print(
                "Credentials Expiration: "
                + format(amazon_client.expiration.astimezone(get_localzone()))
            )

    if config.credential_process:
        amazon_client.print_credential_process()
        config.write_token_cache(amazon_client)
    elif config.print_creds:
        amazon_client.print_export_line()
    elif config.profile:
        config.write(amazon_client)

    config.write_saml_cache()


def main():
    cli_args = sys.argv[1:]
    cli(cli_args)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

import sys
import unittest

from mock import patch, MagicMock

from aws_saml_auth import util


class TestUtilMethods(unittest.TestCase):
    def test_coalesce_no_arguments(self):
        self.assertEqual(util.Util.coalesce(), None)

    def test_coalesce_one_argument(self):
        value = "non_none_value"
        self.assertEqual(util.Util.coalesce(value), value)
        self.assertEqual(util.Util.coalesce(None), None)

    def test_coalesce_two_arguments(self):
        value = "non_none_value"
        self.assertEqual(util.Util.coalesce(value, None), value)
        self.assertEqual(util.Util.coalesce(value, value), value)
        self.assertEqual(util.Util.coalesce(None, value), value)
        self.assertEqual(util.Util.coalesce(None, None), None)

    def test_coalesce_many_arguments(self):
        self.assertEqual(
            util.Util.coalesce(None, "test-01", None, "test-02", None, "test-03"),
            "test-01",
        )
        self.assertEqual(
            util.Util.coalesce("test-01", None, "test-02", None, "test-03", None),
            "test-01",
        )
        self.assertEqual(
            util.Util.coalesce(
                None, None, None, None, None, None, None, None, None, None, "test-01"
            ),
            "test-01",
        )

    def test_unicode_to_string_if_needed_python_3(self):
        if sys.version_info >= (3, 0):
            value_string = "Test String!"
            self.assertIn("str", str(value_string.__class__))
            self.assertEqual(
                util.Util.unicode_to_string_if_needed(value_string), value_string
            )

    def test_unicode_to_string_if_needed_python_2(self):
        if sys.version_info < (3, 0):
            value_string = "Test String!"
            value_unicode = value_string.decode("utf-8")
            self.assertIn("str", str(value_string.__class__))
            self.assertIn("unicode", str(value_unicode.__class__))
            self.assertEqual(
                util.Util.unicode_to_string_if_needed(value_unicode), value_string
            )
            self.assertEqual(
                util.Util.unicode_to_string_if_needed(value_string), value_string
            )

    def test_unicode_to_string_if_needed(self):
        self.assertEqual(util.Util.unicode_to_string_if_needed(None), None)
        self.assertEqual(util.Util.unicode_to_string_if_needed(1234), 1234)
        self.assertEqual(util.Util.unicode_to_string_if_needed("nop"), "nop")



from aws_saml_auth import exit_if_unsupported_python

import unittest
import sys
import mock


class TestPythonFailOnVersion(unittest.TestCase):
    def test_python26(self):

        with mock.patch.object(sys, "version_info") as v_info:
            v_info.major = 2
            v_info.minor = 6

            with self.assertRaises(SystemExit) as cm:
                exit_if_unsupported_python()

            self.assertEqual(cm.exception.code, 1)

    def test_python27(self):
        with mock.patch.object(sys, "version_info") as v_info:
            v_info.major = 2
            v_info.minor = 7

            try:
                exit_if_unsupported_python()
            except SystemExit:
                self.fail(
                    "exit_if_unsupported_python() raised SystemExit unexpectedly!"
                )

    def test_python30(self):
        with mock.patch.object(sys, "version_info") as v_info:
            v_info.major = 3
            v_info.minor = 0

            try:
                exit_if_unsupported_python()
            except SystemExit:
                self.fail(
                    "exit_if_unsupported_python() raised SystemExit unexpectedly!"
                )



#!/usr/bin/env python

import unittest

from aws_saml_auth import parse_args


class TestPythonFailOnVersion(unittest.TestCase):
    def test_no_arguments(self):
        """
        This test case exists to validate the default settings of the args parser.
        Changes that break these checks should be considered for backwards compatibility review.
        :return:
        """
        parser = parse_args([])

        self.assertFalse(parser.redirect_server)
        self.assertEqual(parser.login_url, None)
        self.assertTrue(parser.saml_cache)
        self.assertEqual(parser.saml_assertion, None)
        self.assertFalse(parser.ask_role)
        self.assertFalse(parser.print_creds)
        self.assertFalse(parser.credential_process)
        self.assertTrue(parser.resolve_aliases)

        self.assertEqual(parser.duration, None)
        self.assertEqual(parser.auto_duration, False)
        self.assertEqual(parser.profile, None)
        self.assertEqual(parser.region, None)
        self.assertEqual(parser.role_arn, None)
        self.assertEqual(parser.quiet, False)
        self.assertEqual(parser.account, None)
        self.assertEqual(parser.port, None)

        # Assert the size of the parameter so that new parameters trigger a review of this function
        # and the appropriate defaults are added here to track backwards compatibility in the future.
        self.assertEqual(len(vars(parser)), 17)

    def test_nocache(self):

        parser = parse_args(["--no-saml-cache"])

        self.assertFalse(parser.saml_cache)

    def test_resolvealiases(self):

        parser = parse_args(["--no-resolve-aliases"])

        self.assertFalse(parser.resolve_aliases)

    def test_ask_and_supply_role(self):

        with self.assertRaises(SystemExit):
            parse_args(["-a", "-r", "da-role"])

    def test_invalid_duration(self):
        """
        Should fail parsing a non-int value for `-d`.
        :return:
        """

        with self.assertRaises(SystemExit):
            parse_args(["-d", "abce"])



import unittest
from argparse import Namespace

from mock import call, patch, Mock, MagicMock

import aws_saml_auth


class TestInit(unittest.TestCase):
    def setUp(self):
        pass

    @patch("aws_saml_auth.cli", spec=True)
    def test_main_method_has_no_parameters(self, mock_cli):
        """
        This is the entrypoint for the cli tool, and should require no parameters

        :param mock_cli:
        :return:
        """

        # Function under test
        aws_saml_auth.main()

        self.assertTrue(mock_cli.called)

    # @patch('aws_saml_auth.exit_if_unsupported_python', spec=True)
    # @patch('aws_saml_auth.resolve_config', spec=True)
    # @patch('aws_saml_auth.process_auth', spec=True)
    # def test_main_method_chaining(self, process_auth, resolve_config, exit_if_unsupported_python):
    #
    #     # Create a mock config to be returned from the resolve_config function
    #     mock_config = Mock()
    #     # Inject the mock as the return value from the function
    #     aws_saml_auth.resolve_config.return_value = mock_config
    #
    #     # Function under test
    #     aws_saml_auth.cli([])
    #
    #     self.assertTrue(exit_if_unsupported_python.called)
    #     self.assertTrue(resolve_config.called)
    #     self.assertTrue(process_auth.called)
    #
    #     self.assertEqual([call()], exit_if_unsupported_python.mock_calls)
    #
    #     self.assertEqual([call(Namespace(ask_role=False,
    #                                      keyring=False,
    #                                      disable_u2f=False,
    #                                      duration=None,
    #                                      auto_duration=False,
    #                                      idp_id=None,
    #                                      profile=None,
    #                                      region=None,
    #                                      resolve_aliases=False,
    #                                      role_arn=None,
    #                                      save_failure_html=False,
    #                                      save_saml_flow=False,
    #                                      saml_cache=True,
    #                                      saml_assertion=None,
    #                                      sp_id=None,
    #                                      log_level='warn',
    #                                      print_creds=False,
    #                                      username=None,
    #                                      quiet=False,
    #                                      bg_response=None,
    #                                      account=None))
    #                       ],
    #                      resolve_config.mock_calls)
    #
    #     self.assertEqual([call(Namespace(ask_role=False,
    #                                      keyring=False,
    #                                      disable_u2f=False,
    #                                      duration=None,
    #                                      auto_duration=False,
    #                                      idp_id=None,
    #                                      profile=None,
    #                                      region=None,
    #                                      resolve_aliases=False,
    #                                      role_arn=None,
    #                                      save_failure_html=False,
    #                                      save_saml_flow=False,
    #                                      saml_cache=True,
    #                                      saml_assertion=None,
    #                                      sp_id=None,
    #                                      log_level='warn',
    #                                      print_creds=False,
    #                                      username=None,
    #                                      quiet=False,
    #                                      bg_response=None,
    #                                      account=None),
    #                            mock_config)
    #                       ],
    #                      process_auth.mock_calls)
    #
    # @patch('aws_saml_auth.util', spec=True)
    # @patch('aws_saml_auth.amazon', spec=True)
    # @patch('aws_saml_auth.google', spec=True)
    # def test_process_auth_standard(self, mock_google, mock_amazon, mock_util):
    #
    #     mock_config = Mock()
    #     mock_config.profile = False
    #     mock_config.saml_cache = False
    #     mock_config.keyring = False
    #     mock_config.username = None
    #     mock_config.idp_id = None
    #     mock_config.sp_id = None
    #     mock_config.return_value = None
    #     mock_config.account = None
    #     mock_config.region = None
    #
    #     mock_amazon_client = Mock()
    #     mock_google_client = Mock()
    #
    #     mock_amazon_client.roles = {
    #         'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #         'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #     }
    #
    #     mock_util_obj = MagicMock()
    #     mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
    #     mock_util_obj.get_input = MagicMock(side_effect=["region_input", "input", "input2", "input3"])
    #     mock_util_obj.get_password = MagicMock(return_value="pass")
    #
    #     mock_util.Util = mock_util_obj
    #
    #     mock_amazon_client.resolve_aws_aliases = MagicMock(return_value=[])
    #
    #     mock_amazon.Amazon = MagicMock(return_value=mock_amazon_client)
    #     mock_google.Google = MagicMock(return_value=mock_google_client)
    #
    #     args = aws_saml_auth.parse_args([])
    #
    #     # Method Under Test
    #     aws_saml_auth.process_auth(args, mock_config)
    #
    #     # Assert values collected
    #     self.assertEqual(mock_config.region, "region_input")
    #     self.assertEqual(mock_config.username, "input")
    #     self.assertEqual(mock_config.idp_id, "input2")
    #     self.assertEqual(mock_config.sp_id, "input3")
    #     self.assertEqual(mock_config.password, "pass")
    #     self.assertEqual(mock_config.provider, "da_provider")
    #     self.assertEqual(mock_config.role_arn, "da_role")
    #
    #     # Assert calls occur
    #     self.assertEqual([call.Util.get_input('AWS Region: '),
    #                       call.Util.get_input('Google username: '),
    #                       call.Util.get_input('Google IDP ID: '),
    #                       call.Util.get_input('Google SP ID: '),
    #                       call.Util.get_password('Google Password: '),
    #                       call.Util.pick_a_role({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])],
    #                      mock_util.mock_calls)
    #
    #     self.assertEqual([call.do_login(), call.parse_saml()],
    #                      mock_google_client.mock_calls)
    #
    #     self.assertEqual([call.raise_if_invalid()],
    #                      mock_config.mock_calls)
    #
    #     self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #                             })],
    #                      mock_amazon_client.resolve_aws_aliases.mock_calls)
    #
    #     self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])
    #                       ], mock_util_obj.pick_a_role.mock_calls)
    #
    # @patch('aws_saml_auth.util', spec=True)
    # @patch('aws_saml_auth.amazon', spec=True)
    # @patch('aws_saml_auth.google', spec=True)
    # def test_process_auth_print_creds(self, mock_google, mock_amazon, mock_util):
    #     mock_config = Mock()
    #     mock_config.profile = False
    #     mock_config.saml_cache = False
    #     mock_config.keyring = False
    #     mock_config.username = None
    #     mock_config.idp_id = None
    #     mock_config.sp_id = None
    #     mock_config.return_value = None
    #     mock_config.print_creds = True
    #     mock_config.account = None
    #
    #     mock_amazon_client = Mock()
    #     mock_google_client = Mock()
    #
    #     mock_amazon_client.roles = {
    #         'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #         'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #     }
    #
    #     mock_util_obj = MagicMock()
    #     mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
    #     mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
    #     mock_util_obj.get_password = MagicMock(return_value="pass")
    #
    #     mock_util.Util = mock_util_obj
    #
    #     mock_amazon_client.resolve_aws_aliases = MagicMock(return_value=[])
    #     mock_amazon_client.print_export_line = Mock()
    #
    #     mock_amazon.Amazon = MagicMock(return_value=mock_amazon_client)
    #     mock_google.Google = MagicMock(return_value=mock_google_client)
    #
    #     args = aws_saml_auth.parse_args([])
    #
    #     # Method Under Test
    #     aws_saml_auth.process_auth(args, mock_config)
    #
    #     # Assert values collected
    #     self.assertEqual(mock_config.username, "input")
    #     self.assertEqual(mock_config.idp_id, "input2")
    #     self.assertEqual(mock_config.sp_id, "input3")
    #     self.assertEqual(mock_config.password, "pass")
    #     self.assertEqual(mock_config.provider, "da_provider")
    #     self.assertEqual(mock_config.role_arn, "da_role")
    #
    #     # Assert calls occur
    #     self.assertEqual([call.Util.get_input('Google username: '),
    #                       call.Util.get_input('Google IDP ID: '),
    #                       call.Util.get_input('Google SP ID: '),
    #                       call.Util.get_password('Google Password: '),
    #                       call.Util.pick_a_role({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'},
    #                                             [])],
    #                      mock_util.mock_calls)
    #
    #     self.assertEqual([call.do_login(), call.parse_saml()],
    #                      mock_google_client.mock_calls)
    #
    #     self.assertEqual([call.raise_if_invalid()],
    #                      mock_config.mock_calls)
    #
    #     self.assertEqual(
    #         [call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #                })],
    #         mock_amazon_client.resolve_aws_aliases.mock_calls)
    #
    #     self.assertEqual(
    #         [call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])
    #          ], mock_util_obj.pick_a_role.mock_calls)
    #
    #     self.assertEqual([call()],
    #                      mock_amazon_client.print_export_line.mock_calls)
    #
    # @patch('aws_saml_auth.util', spec=True)
    # @patch('aws_saml_auth.amazon', spec=True)
    # @patch('aws_saml_auth.google', spec=True)
    # def test_process_auth_specified_role(self, mock_google, mock_amazon, mock_util):
    #
    #     mock_config = Mock()
    #     mock_config.saml_cache = False
    #     mock_config.keyring = False
    #     mock_config.username = None
    #     mock_config.idp_id = None
    #     mock_config.sp_id = None
    #     mock_config.return_value = None
    #
    #     mock_config.role_arn = 'arn:aws:iam::123456789012:role/admin'
    #     mock_config.ask_role = False
    #
    #     mock_amazon_client = Mock()
    #     mock_google_client = Mock()
    #
    #     mock_amazon_client.roles = {
    #         'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #         'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #     }
    #
    #     mock_util_obj = MagicMock()
    #     mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
    #     mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
    #     mock_util_obj.get_password = MagicMock(return_value="pass")
    #
    #     mock_util.Util = mock_util_obj
    #
    #     mock_amazon_client.resolve_aws_aliases = MagicMock(return_value=[])
    #
    #     mock_amazon.Amazon = MagicMock(return_value=mock_amazon_client)
    #     mock_google.Google = MagicMock(return_value=mock_google_client)
    #
    #     args = aws_saml_auth.parse_args([])
    #
    #     # Method Under Test
    #     aws_saml_auth.process_auth(args, mock_config)
    #
    #     # Assert values collected
    #     self.assertEqual(mock_config.username, "input")
    #     self.assertEqual(mock_config.idp_id, "input2")
    #     self.assertEqual(mock_config.sp_id, "input3")
    #     self.assertEqual(mock_config.password, "pass")
    #     self.assertEqual(mock_config.provider, "arn:aws:iam::123456789012:saml-provider/GoogleApps")
    #     self.assertEqual(mock_config.role_arn, "arn:aws:iam::123456789012:role/admin")
    #
    #     # Assert calls occur
    #     self.assertEqual([call.Util.get_input('Google username: '),
    #                       call.Util.get_input('Google IDP ID: '),
    #                       call.Util.get_input('Google SP ID: '),
    #                       call.Util.get_password('Google Password: ')],
    #                      mock_util.mock_calls)
    #
    #     self.assertEqual([call.do_login(), call.parse_saml()],
    #                      mock_google_client.mock_calls)
    #
    #     self.assertEqual([call.raise_if_invalid(),
    #                       call.write(mock_amazon_client)],
    #                      mock_config.mock_calls)
    #
    #     self.assertEqual([],
    #                      mock_amazon_client.resolve_aws_aliases.mock_calls)
    #
    #     self.assertEqual([],
    #                      mock_util_obj.pick_a_role.mock_calls)
    #
    # @patch('aws_saml_auth.util', spec=True)
    # @patch('aws_saml_auth.amazon', spec=True)
    # @patch('aws_saml_auth.google', spec=True)
    # def test_process_auth_dont_resolve_alias(self, mock_google, mock_amazon, mock_util):
    #
    #     mock_config = Mock()
    #     mock_config.saml_cache = False
    #     mock_config.resolve_aliases = False
    #     mock_config.username = None
    #     mock_config.idp_id = None
    #     mock_config.sp_id = None
    #     mock_config.return_value = None
    #     mock_config.keyring = False
    #     mock_config.account = None
    #
    #     mock_amazon_client = Mock()
    #     mock_google_client = Mock()
    #
    #     mock_amazon_client.roles = {
    #         'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #         'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #     }
    #
    #     mock_util_obj = MagicMock()
    #     mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
    #     mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
    #     mock_util_obj.get_password = MagicMock(return_value="pass")
    #
    #     mock_util.Util = mock_util_obj
    #
    #     mock_amazon_client.resolve_aws_aliases = MagicMock(return_value=[])
    #
    #     mock_amazon.Amazon = MagicMock(return_value=mock_amazon_client)
    #     mock_google.Google = MagicMock(return_value=mock_google_client)
    #
    #     args = aws_saml_auth.parse_args([])
    #
    #     # Method Under Test
    #     aws_saml_auth.process_auth(args, mock_config)
    #
    #     # Assert values collected
    #     self.assertEqual(mock_config.username, "input")
    #     self.assertEqual(mock_config.idp_id, "input2")
    #     self.assertEqual(mock_config.sp_id, "input3")
    #     self.assertEqual(mock_config.password, "pass")
    #     self.assertEqual(mock_config.provider, "da_provider")
    #     self.assertEqual(mock_config.role_arn, "da_role")
    #     self.assertEqual(mock_config.account, None)
    #
    #     # Assert calls occur
    #     self.assertEqual([call.Util.get_input('Google username: '),
    #                       call.Util.get_input('Google IDP ID: '),
    #                       call.Util.get_input('Google SP ID: '),
    #                       call.Util.get_password('Google Password: '),
    #                       call.Util.pick_a_role({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'})],
    #                      mock_util.mock_calls)
    #
    #     self.assertEqual([call.do_login(), call.parse_saml()],
    #                      mock_google_client.mock_calls)
    #
    #     self.assertEqual([call.raise_if_invalid(),
    #                       call.write(mock_amazon_client)],
    #                      mock_config.mock_calls)
    #
    #     self.assertEqual([],
    #                      mock_amazon_client.resolve_aws_aliases.mock_calls)
    #
    #     self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'})
    #                       ], mock_util_obj.pick_a_role.mock_calls)
    #
    # @patch('aws_saml_auth.util', spec=True)
    # @patch('aws_saml_auth.amazon', spec=True)
    # @patch('aws_saml_auth.google', spec=True)
    # def test_process_auth_with_profile(self, mock_google, mock_amazon, mock_util):
    #
    #     mock_config = Mock()
    #     mock_config.saml_cache = False
    #     mock_config.keyring = False
    #     mock_config.username = None
    #     mock_config.idp_id = None
    #     mock_config.sp_id = None
    #     mock_config.profile = "blart"
    #     mock_config.return_value = None
    #     mock_config.role_arn = 'arn:aws:iam::123456789012:role/admin'
    #     mock_config.account = None
    #
    #     mock_amazon_client = Mock()
    #     mock_google_client = Mock()
    #
    #     mock_amazon_client.roles = {
    #         'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #         'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #     }
    #
    #     mock_util_obj = MagicMock()
    #     mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
    #     mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
    #     mock_util_obj.get_password = MagicMock(return_value="pass")
    #
    #     mock_util.Util = mock_util_obj
    #
    #     mock_amazon_client.resolve_aws_aliases = MagicMock(return_value=[])
    #
    #     mock_amazon.Amazon = MagicMock(return_value=mock_amazon_client)
    #     mock_google.Google = MagicMock(return_value=mock_google_client)
    #
    #     args = aws_saml_auth.parse_args([])
    #
    #     # Method Under Test
    #     aws_saml_auth.process_auth(args, mock_config)
    #
    #     # Assert values collected
    #     self.assertEqual(mock_config.username, "input")
    #     self.assertEqual(mock_config.idp_id, "input2")
    #     self.assertEqual(mock_config.sp_id, "input3")
    #     self.assertEqual(mock_config.password, "pass")
    #     self.assertEqual(mock_config.provider, "da_provider")
    #     self.assertEqual(mock_config.role_arn, "da_role")
    #
    #     # Assert calls occur
    #     self.assertEqual([call.Util.get_input('Google username: '),
    #                       call.Util.get_input('Google IDP ID: '),
    #                       call.Util.get_input('Google SP ID: '),
    #                       call.Util.get_password('Google Password: '),
    #                       call.Util.pick_a_role({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])],
    #                      mock_util.mock_calls)
    #
    #     self.assertEqual([call.do_login(), call.parse_saml()],
    #                      mock_google_client.mock_calls)
    #
    #     self.assertEqual([call.raise_if_invalid(),
    #                       call.write(mock_amazon_client)],
    #                      mock_config.mock_calls)
    #
    #     self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #                             })],
    #                      mock_amazon_client.resolve_aws_aliases.mock_calls)
    #
    #     self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])
    #                       ], mock_util_obj.pick_a_role.mock_calls)
    #
    # @patch('aws_saml_auth.util', spec=True)
    # @patch('aws_saml_auth.amazon', spec=True)
    # @patch('aws_saml_auth.saml', spec=True)
    # def test_process_auth_with_saml_cache(self, mock_google, mock_amazon, mock_util):
    #
    #     mock_config = Mock()
    #     mock_config.saml_cache = True
    #     mock_config.login_url = 'sample_login_url'
    #     mock_config.return_value = None
    #     mock_config.role_arn = 'arn:aws:iam::123456789012:role/admin'
    #     mock_config.account = None
    #
    #     mock_amazon_client = Mock()
    #     mock_saml_client = Mock()
    #
    #     mock_amazon_client.roles = {
    #         'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #         'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #     }
    #
    #     mock_util_obj = MagicMock()
    #     mock_util_obj.pick_a_role = MagicMock(return_value=("da_role", "da_provider"))
    #     mock_util_obj.get_input = MagicMock(side_effect=["input", "input2", "input3"])
    #
    #     mock_util.Util = mock_util_obj
    #
    #     mock_amazon_client.resolve_aws_aliases = MagicMock(return_value=[])
    #
    #     mock_amazon.Amazon = MagicMock(return_value=mock_amazon_client)
    #     mock_google.Saml = MagicMock(return_value=mock_saml_client)
    #
    #     args = aws_saml_auth.parse_args([])
    #
    #     # Method Under Test
    #     aws_saml_auth.process_auth(args, mock_config)
    #
    #     # Assert values collected
    #     self.assertEqual(mock_config.login_url, 'sample_login_url')
    #     self.assertEqual(mock_config.provider, "da_provider")
    #     self.assertEqual(mock_config.role_arn, "da_role")
    #
    #     # Assert calls occur
    #     self.assertEqual([call.Util.pick_a_role({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])],
    #                      mock_util.mock_calls)
    #
    #     # Cache means no google calls
    #     self.assertEqual([],
    #                      mock_saml_client.mock_calls)
    #
    #     self.assertEqual([call.write(mock_amazon_client)],
    #                      mock_config.mock_calls)
    #
    #     self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'
    #                             })],
    #                      mock_amazon_client.resolve_aws_aliases.mock_calls)
    #
    #     self.assertEqual([call({'arn:aws:iam::123456789012:role/read-only': 'arn:aws:iam::123456789012:saml-provider/GoogleApps',
    #                             'arn:aws:iam::123456789012:role/admin': 'arn:aws:iam::123456789012:saml-provider/GoogleApps'}, [])
    #                       ], mock_util_obj.pick_a_role.mock_calls)



#!/usr/bin/env python

import unittest

from aws_saml_auth import configuration


class TestConfigurationMethods(unittest.TestCase):
    def test_config_profile(self):
        self.assertEqual(
            configuration.Configuration.config_profile("default"), "default"
        )
        self.assertEqual(
            configuration.Configuration.config_profile("DEFAULT"), "DEFAULT"
        )
        self.assertEqual(
            configuration.Configuration.config_profile("testing"), "profile testing"
        )
        self.assertEqual(
            configuration.Configuration.config_profile(None), "profile None"
        )
        self.assertEqual(
            configuration.Configuration.config_profile(123456), "profile 123456"
        )

    def test_duration_invalid_values(self):
        # Duration must be an integer
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.duration = "bad_type"
        c.region = "sample_region"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected duration to be an integer.", str(e.exception))

        # Duration can not be negative
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.duration = -1
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn(
            "Expected duration to be greater than or equal to 900.", str(e.exception)
        )

        # Duration can not be greater than MAX_DURATION
        valid = configuration.Configuration()
        valid.login_url = "sample_login_url"
        valid.duration = 900
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.duration = valid.max_duration + 1
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn(
            "Expected duration to be less than or equal to max_duration",
            str(e.exception),
        )

    def test_duration_valid_values(self):
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.duration = 900
        self.assertEqual(c.duration, 900)
        c.raise_if_invalid()
        c.duration = c.max_duration
        self.assertEqual(c.duration, c.max_duration)
        c.raise_if_invalid()
        c.duration = c.max_duration - 1
        self.assertEqual(c.duration, c.max_duration - 1)
        c.raise_if_invalid()

    def test_duration_defaults_to_max_duration(self):
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        self.assertEqual(c.duration, c.max_duration)
        c.raise_if_invalid()

    def test_ask_role_invalid_values(self):
        # ask_role must be a boolean
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.ask_role = "bad_value"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected ask_role to be a boolean.", str(e.exception))

    def test_ask_role_valid_values(self):
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.ask_role = True
        self.assertTrue(c.ask_role)
        c.raise_if_invalid()
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.ask_role = False
        self.assertFalse(c.ask_role)
        c.raise_if_invalid()

    def test_ask_role_optional(self):
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "samlple_login_url"
        self.assertFalse(c.ask_role)
        c.raise_if_invalid()

    def test_login_url_invalid_values(self):
        # sp_id must not be None
        c = configuration.Configuration()
        c.region = "sample_region"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn(
            "Expected login_url to be set to non-None value.", str(e.exception)
        )

    def test_profile_defaults_to_sts(self):
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        self.assertEqual(c.profile, "sts")
        c.raise_if_invalid()

    def test_profile_invalid_values(self):
        # profile must be a string
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.profile = 123456
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected profile to be a string.", str(e.exception))

    def test_profile_valid_values(self):
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.profile = "default"
        self.assertEqual(c.profile, "default")
        c.raise_if_invalid()
        c.profile = "sts"
        self.assertEqual(c.profile, "sts")
        c.raise_if_invalid()

    def test_profile_defaults(self):
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        self.assertEqual(c.profile, "sts")
        c.raise_if_invalid()

    def test_region_invalid_values(self):
        # region must be a string
        c = configuration.Configuration()
        c.login_url = "sample_login_url"
        c.region = 1234
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected region to be a string.", str(e.exception))

    def test_region_valid_values(self):
        c = configuration.Configuration()
        c.login_url = "sample_login_url"
        c.region = "us-east-1"
        self.assertEqual(c.region, "us-east-1")
        c.raise_if_invalid()
        c.region = "us-west-2"
        self.assertEqual(c.region, "us-west-2")
        c.raise_if_invalid()

    def test_region_defaults_to_none(self):
        c = configuration.Configuration()
        c.login_url = "sample_login_url"
        self.assertEqual(c.region, None)
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected region to be a string.", str(e.exception))

    def test_role_arn_invalid_values(self):
        # role_arn must be a string
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.role_arn = 1234
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected role_arn to be None or a string.", str(e.exception))

        # role_arn be a arn-looking string
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.role_arn = "bad_string"
        with self.assertRaises(AssertionError) as e:
            c.raise_if_invalid()
        self.assertIn("Expected role_arn to contain 'arn:aws:iam::'", str(e.exception))

    def test_role_arn_is_optional(self):
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        self.assertIsNone(c.role_arn)
        c.raise_if_invalid()

    def test_role_arn_valid_values(self):
        c = configuration.Configuration()
        c.region = "sample_region"
        c.login_url = "sample_login_url"
        c.role_arn = "arn:aws:iam::some_arn_1"
        self.assertEqual(c.role_arn, "arn:aws:iam::some_arn_1")
        c.raise_if_invalid()
        c.role_arn = "arn:aws:iam::some_other_arn_2"
        self.assertEqual(c.role_arn, "arn:aws:iam::some_other_arn_2")
        c.raise_if_invalid()



#!/usr/bin/env python

import unittest
import mock

from aws_saml_auth import amazon
from aws_saml_auth import configuration
from os import path
import os


class TestAmazon(unittest.TestCase):
    @property
    def valid_config(self):
        return configuration.Configuration(
            idp_id="IDPID",
            sp_id="SPID",
            username="user@example.com",
            password="hunter2",
        )

    def read_local_file(self, filename):
        here = path.abspath(path.dirname(__file__))
        with open(path.join(here, filename)) as fp:
            return fp.read().encode("utf-8")

    def test_sts_client(self):
        a = amazon.Amazon(self.valid_config, "dummy-encoded-saml")
        self.assertEqual(str(a.sts_client.__class__), "<class 'botocore.client.STS'>")

    def test_role_extraction(self):
        saml_xml = self.read_local_file("valid-response.xml")
        a = amazon.Amazon(self.valid_config, saml_xml)
        self.assertIsInstance(a.roles, dict)
        list_of_testing_roles = [
            "arn:aws:iam::123456789012:role/admin",
            "arn:aws:iam::123456789012:role/read-only",
            "arn:aws:iam::123456789012:role/test",
        ]
        self.assertEqual(sorted(list(a.roles.keys())), sorted(list_of_testing_roles))

    def test_role_extraction_too_many_commas(self):
        # See https://github.com/cevoaustralia/aws-google-auth/issues/12
        saml_xml = self.read_local_file("too-many-commas.xml")
        a = amazon.Amazon(self.valid_config, saml_xml)
        self.assertIsInstance(a.roles, dict)
        list_of_testing_roles = [
            "arn:aws:iam::123456789012:role/admin",
            "arn:aws:iam::123456789012:role/read-only",
            "arn:aws:iam::123456789012:role/test",
        ]
        self.assertEqual(sorted(list(a.roles.keys())), sorted(list_of_testing_roles))

    def test_invalid_saml_too_soon(self):
        saml_xml = self.read_local_file("saml-response-too-soon.xml")
        self.assertFalse(amazon.Amazon.is_valid_saml_assertion(saml_xml))

    def test_invalid_saml_too_late(self):
        saml_xml = self.read_local_file("saml-response-too-late.xml")
        self.assertFalse(amazon.Amazon.is_valid_saml_assertion(saml_xml))

    def test_invalid_saml_expired_before_valid(self):
        saml_xml = self.read_local_file("saml-response-expired-before-valid.xml")
        self.assertFalse(amazon.Amazon.is_valid_saml_assertion(saml_xml))

    def test_invalid_saml_bad_input(self):
        self.assertFalse(amazon.Amazon.is_valid_saml_assertion(None))
        self.assertFalse(amazon.Amazon.is_valid_saml_assertion("Malformed Base64"))
        self.assertFalse(amazon.Amazon.is_valid_saml_assertion(123456))
        self.assertFalse(amazon.Amazon.is_valid_saml_assertion(""))
        self.assertFalse(
            amazon.Amazon.is_valid_saml_assertion("QmFkIFhNTA==")
        )  # Bad XML

    def test_valid_saml(self):
        saml_xml = self.read_local_file("saml-response-no-expire.xml")
        self.assertTrue(amazon.Amazon.is_valid_saml_assertion(saml_xml))

    @mock.patch.dict(
        os.environ, {"AWS_PROFILE": "xxx-xxxx", "DEFAULT_AWS_PROFILE": "blart"}
    )
    def test_sts_client_with_invalid_profile(self):
        a = amazon.Amazon(self.valid_config, "dummy-encoded-saml")

        self.assertIsNotNone(a.sts_client)

        self.assertEqual("xxx-xxxx", os.environ["AWS_PROFILE"])
        self.assertEqual("blart", os.environ["DEFAULT_AWS_PROFILE"])



#!/usr/bin/env python

from __future__ import print_function

import getpass
import os
import sys
from collections import OrderedDict
from urllib.parse import parse_qs
from cgi import parse_header, parse_multipart

from six.moves import input
from tabulate import tabulate


class Util:
    @staticmethod
    def get_input(prompt):
        return input(prompt)

    @staticmethod
    def pick_a_role(roles, aliases=None, account=None):
        if account:
            filtered_roles = {
                role: principal
                for role, principal in roles.items()
                if (account in role)
            }
        else:
            filtered_roles = roles

        if aliases:
            enriched_roles = {}
            for role, principal in filtered_roles.items():
                enriched_roles[role] = [
                    aliases[role.split(":")[4]],
                    role.split("role/")[1],
                    principal,
                ]
            enriched_roles = OrderedDict(
                sorted(enriched_roles.items(), key=lambda t: (t[1][0], t[1][1]))
            )

            ordered_roles = OrderedDict()
            for role, role_property in enriched_roles.items():
                ordered_roles[role] = role_property[2]

            enriched_roles_tab = []
            for i, (role, role_property) in enumerate(enriched_roles.items()):
                enriched_roles_tab.append([i + 1, role_property[0], role_property[1]])

            while True:
                print(
                    tabulate(
                        enriched_roles_tab,
                        headers=["No", "AWS account", "Role"],
                    )
                )
                prompt = "Type the number (1 - {:d}) of the role to assume: ".format(
                    len(enriched_roles)
                )
                choice = Util.get_input(prompt)

                try:
                    return list(ordered_roles.items())[int(choice) - 1]
                except (IndexError, ValueError):
                    print("Invalid choice, try again.")
        else:
            while True:
                for i, role in enumerate(filtered_roles):
                    print("[{:>3d}] {}".format(i + 1, role))

                prompt = "Type the number (1 - {:d}) of the role to assume: ".format(
                    len(filtered_roles)
                )
                choice = Util.get_input(prompt)

                try:
                    return list(filtered_roles.items())[int(choice) - 1]
                except (IndexError, ValueError):
                    print("Invalid choice, try again.")

    @staticmethod
    def touch(file_name, mode=0o600):
        flags = os.O_CREAT | os.O_APPEND
        with os.fdopen(os.open(file_name, flags, mode)) as f:
            try:
                os.utime(file_name, None)
            finally:
                f.close()

    # This method returns the first non-None value in args. If all values are
    # None, None will be returned. If there are no arguments, None will be
    # returned.
    @staticmethod
    def coalesce(*args):
        for _, value in enumerate(args):
            if value is not None:
                return value
        return None

    @staticmethod
    def unicode_to_string_if_needed(object):
        if "unicode" in str(object.__class__):
            return object.encode("utf-8")
        else:
            return object

    @staticmethod
    def parse_post(handler):
        if "content-type" not in handler.headers:
            return {}
        ctype, pdict = parse_header(handler.headers["content-type"])
        if ctype == "multipart/form-data":
            postvars = parse_multipart(handler.rfile, pdict)
        elif ctype == "application/x-www-form-urlencoded":
            length = int(handler.headers["content-length"])
            postvars = parse_qs(
                handler.rfile.read(length).decode("utf-8"), keep_blank_values=1
            )
        else:
            postvars = {}
        return postvars
